﻿using System;
using System.Threading;

// DataPodprocesu
public class DataPodprocesu
{
    public RegisteredWaitHandle Handle = null;
    public string Další = "";
}

public class Signalizace
{
    public static void Main(string[] args)
    {
        // AutoResetEvent slouží k signalizaci,
        // že se má něco stát (sppustit podproces)
        AutoResetEvent ev = new AutoResetEvent(false);

        DataPodprocesu dataPodprocesu = new DataPodprocesu();
        dataPodprocesu.Další = "Příklad";
        // dataPodprocesu obsahují identifikaci čekajícího podprocesu
        // (handle) vrácenou meotodou RegisterWaitForSingleObject.  
        // To umožňuje odregistrovat tuto metodu

        System.Console.WriteLine("Registrace");
        dataPodprocesu.Handle = ThreadPool.RegisterWaitForSingleObject(
            ev,
            new WaitOrTimerCallback(ČekajícíProcedura),
            dataPodprocesu,
            -1,
            false);
        System.Console.WriteLine("Po registraci");

        // Hlavní podproces počká 3 vteřiny, aby předvedl,
        // že se podproces po uplunutí dané doby spustí, a pak signalizuje.
        Thread.Sleep(3100);
        Console.WriteLine("Hlavní podproces signalizuje.");
        ev.Set();

        // Hlavní podproces spí; to poskytne čas k provedení podprocesu 
        Thread.Sleep(1000);
        // Nemáme možnost volat Thread.Join; řešení viz další příklad
    }

    // Čekající procedura se spustí po uplunutí zadaného intervalu,
    // nebo při signalizaci pomocí parametru WaitHandle (zde AutoResetEvent).
    // tato procedura zruší svou registraci při první signalizaci.
    public static void ČekajícíProcedura(object stav, bool časUplynul)
    {
        // Parametr nutno přetypovat
        DataPodprocesu dataPodprocesu = (DataPodprocesu)stav;

        string důvod = "Vypršel čas";
        if (!časUplynul)
        {
            důvod = "Signalizace";
            if (dataPodprocesu.Handle != null)
                dataPodprocesu.Handle.Unregister(null);
        }


        Console.WriteLine("ČekajícíProcedura( {0} ) spuštěna v podprocesu {1}; důvod volání: {2}.",
            dataPodprocesu.Další,
            Thread.CurrentThread.GetHashCode().ToString(),
            důvod
        );
    }
}


