using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace serial
{
    [Serializable]
    public class Hrana
    {
        int potek, konec;
        public Hrana(int potek, int konec)
        {
            this.potek = potek;
            this.konec = konec;
        }
        public int Potek
        {
            get{ return potek; }
        }
        public int Konec
        {
            get{ return konec; }
        }
        public override bool Equals(object obj)
    	{
          if (!obj.GetType().Equals(typeof(Hrana))) return false;
          Hrana hrana = (Hrana)obj;
          return (hrana.potek == this.potek) && (hrana.konec == this.konec);
    	}
        public override string ToString()
        {
            return "<" + potek + ", " + konec +">";
        }
    }

    [Serializable]
    public class Graf
    {
        private int poetUzl;
        private List<Hrana> seznamHran = new List<Hrana>();
        public Graf(int poetUzl)
        {
            this.poetUzl = poetUzl;
        }
        public void PidejHranu(Hrana hrana)
        {
            if((hrana.Potek > poetUzl) || (hrana.Konec > poetUzl))
                throw new ArgumentOutOfRangeException("Takov uzel nen");
            if (seznamHran.IndexOf(hrana) < 0)
                seznamHran.Add(hrana);
        }
        public void VypiGraf(TextWriter kam)
        {
            foreach (Hrana hrana in seznamHran)
                Console.WriteLine(hrana);
        }
        public void Ulo(string kam)
        {
            using (FileStream proud = new FileStream(kam, FileMode.Create, FileAccess.Write))
            {
                BinaryFormatter serializtor = new BinaryFormatter();
                serializtor.Serialize(proud, this);
            }
        }

        public static Graf Nati(string odkud)
        {
            using (FileStream proud = new FileStream(odkud, FileMode.Open, FileAccess.Read))
            {
                BinaryFormatter serializtor = new BinaryFormatter();
                return serializtor.Deserialize(proud) as Graf;
            }
        }
    }
    class Program
    {
        static void Main(string[] args)
        {
            Graf g = new Graf(4);
            g.PidejHranu(new Hrana(1, 4));
            g.PidejHranu(new Hrana(2, 3));
            g.PidejHranu(new Hrana(4, 1));
            g.VypiGraf(Console.Out);
            g.Ulo("data.dta");            // Uloen pomoc serializace
	    g = null;
            g = Graf.Nati("data.dta"); 
            if (g != null)                 // Naten serializovanho grafu
                g.VypiGraf(Console.Out);
        }
    }
}
