﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Text;

namespace linqToSql
{
    [Table(Name = "Kniha")]
    class Kniha
    {
        [Column(IsPrimaryKey = true)]
        public int Id { get; set; }
        [Column]
        public string Titul { get; set; }
        [Column]
        public int Cena { get; set; }
        [Column]
        public int PočetAutorů { get; set; }
    }

    [Table(Name = "Autor")]
    class Autor
    {
        [Column(IsPrimaryKey = true)]
        public int ID { get; set; }
        [Column]
        public string Příjmení { get; set; }
        [Column]
        public string Jméno { get; set; }
    }

    [Table(Name = "Autorství")]
    class Autorství
    {
        [Column]
        public int ID_Autora { get; set; }
        [Column]
        public int ID_Knihy { get; set; }
    }


    class Program
    {
        static void Main(string[] args)
        {
            string jméno = "Singh";
            VypišKnihyDanéhoAutora(jméno);
            VypišKnihyDanéhoAutoraJinak(jméno);
            VypišKnihyDanéhoAutoraJeštěJinak(jméno);
        }

        private static void VypišKnihyDanéhoAutora(string příjmení)
        {
            DataContext kontext = new DataContext(Properties.Settings.Default.připojení);
            Table<Autor> autor = kontext.GetTable<Autor>();
            Table<Kniha> kniha = kontext.GetTable<Kniha>();
            Table<Autorství> autorství = kontext.GetTable<Autorství>();
            var dotaz = from x in autor
                        join y in autorství on x.ID equals y.ID_Autora
                        join z in kniha on y.ID_Knihy equals z.Id
                        where x.Příjmení == příjmení
                        select x.Jméno + " " + x.Příjmení + ": " + z.Titul;
            foreach (var w in dotaz)
                Console.WriteLine(w);
        }

        private static void VypišKnihyDanéhoAutoraJinak(string příjmení)
        {
            DataContext kontext = new DataContext(Properties.Settings.Default.připojení);
            Table<Autor> autor = kontext.GetTable<Autor>();
            Table<Kniha> kniha = kontext.GetTable<Kniha>();
            Table<Autorství> autorství = kontext.GetTable<Autorství>();
            var dotaz = from x in autor
                        from y in autorství 
                        from z in kniha 
                        where (x.ID == y.ID_Autora) && (y.ID_Knihy == z.Id) && (x.Příjmení == příjmení) 
                        select x.Jméno + " " + x.Příjmení + ": " + z.Titul;
            foreach (var w in dotaz)
                Console.WriteLine(w);
        }

        private static void VypišKnihyDanéhoAutoraJeštěJinak(string příjmení)
        {
            DataContext kontext = new DataContext(Properties.Settings.Default.připojení);
            Table<Autor> autor = kontext.GetTable<Autor>();
            Table<Kniha> kniha = kontext.GetTable<Kniha>();
            Table<Autorství> autorství = kontext.GetTable<Autorství>();
            var dotaz = from x in autor
                        join y in autorství on x.ID equals y.ID_Autora
                        join z in kniha on y.ID_Knihy equals z.Id into fšechno
                        from w in fšechno
                        where x.Příjmení == příjmení
                        select x.Příjmení + " " + w.Titul;
            foreach (var w in dotaz)
                Console.WriteLine(w);
        }
    }
}
